package board;

import objects.Token;

public class Grid<T> implements IGrid<T> {
    private final T[][] grid;
    private final int width;
    private final int height;
    private final T empty;

    public Grid(int width, int height, T object) {
        T[][] list = (T[][])new Object[width][height];
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < height; j++) {
                list[i][j] = object;
            }
        }
        grid = list;
        this.empty = object;
        this.width = width;
        this.height = height;
    }

    public void flush() {
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < height; j++) {
                grid[i][j] = empty;
            }
        }
    }

    public boolean isFull() {
        int empty = 0;
        for (T[] list : grid) {
            for (T item : list) {
                if (this.empty.getClass().isInstance(item)) {
                    empty++;
                }
            }
        }
        return empty < 1;
    }

    @Override
    public int getTopObject(int column) {
        for (int j = this.height - 1; j > -1; j--) {
            if (grid[column][j] instanceof Token) {
                return j;
            }
        }
        return -1;
    }

    @Override
    public int getHeight() {
        return this.height;
    }

    @Override
    public int getWidth() {
        return this.width;
    }

    @Override
    public boolean drop(int i, T element) {
        if (i > width || i < 0) {
            throw new IllegalArgumentException("Index out of bounds.");
        }
        for (int j = 0; j < this.height; j++) {
            if (this.empty.getClass().isInstance(grid[i][j])) {
                grid[i][j] = element;
                return true;
            }
        }
        return false;
    }

    @Override
    public void set(int x, int y, T element) {
        if (x > width || y > height || x < 0 || y < 0) {
            throw new IllegalArgumentException("Index out of bounds.");
        }
        grid[x][y] = element;
    }

    @Override
    public T get(int x, int y) {
        if (y >= height || x >= width || x < 0 || y < 0) {
            throw new IllegalArgumentException("Index out of bound.");
        }
        return grid[x][y];
    }

    @Override
    public IGrid<T> copy() {
        Grid<T> newGrid = new Grid<>(width, height, empty);
        System.arraycopy(grid, 0, newGrid.grid, 0, width);
        return newGrid;
    }
}
